import sinon from 'sinon';
import { get as getBaseUrl } from '../services/baseUrl.js';
import { GET } from '../constants/constants.js';

import {
  request,
  getRequest,
  postRequest,
  putRequest,
  patchRequest,
} from './request.js';

const mockUrlMaker = params => `v0/transactions/brands&${params.affiliationCode}`;

const mockUrl = 'v0/transactions/brands&111111111';

const mockRequestParams = { affiliationCode: '111111111' };

const defaultBaseUrl = getBaseUrl();

const defaultRequestParams = {
  method: GET,
  headers: {},
};

describe('Request Helper', () => {
  it('Should call fetchMaker without any arguments.', () => {
    const fetchMaker = sinon.spy();

    request(undefined, undefined, undefined, fetchMaker)();

    expect(fetchMaker)
      .to
      .have
      .been.calledWith(defaultBaseUrl, defaultRequestParams);
  });

  it('Should call fetchMaker passing only an url string.', () => {
    const fetchMaker = sinon.spy();

    request(mockUrl, undefined, undefined, fetchMaker)();

    expect(fetchMaker)
      .to
      .have
      .been.calledWith(`${defaultBaseUrl}${mockUrl}`, defaultRequestParams);
  });

  it('Should call fetchMaker passing only request parameters.', () => {
    const fetchMaker = sinon.spy();

    request(undefined, undefined, undefined, fetchMaker)(mockRequestParams);

    expect(fetchMaker)
      .to
      .have
      .been.calledWith(defaultBaseUrl, defaultRequestParams);
  });

  it('Should call fetchMaker an urlMaker function and ' +
    'request parameters.', () => {
    const fetchMaker = sinon.spy();

    request(mockUrlMaker, undefined, undefined, fetchMaker)(mockRequestParams);

    expect(fetchMaker)
      .to
      .have
      .been.calledWith(`${defaultBaseUrl}${mockUrl}`, defaultRequestParams);
  });

  it('Should call fetchMaker with a custom base url and ' +
    'an urlMaker function.', () => {
    const fetchMaker = sinon.spy();

    request(mockUrlMaker, undefined, undefined, fetchMaker)(
      'test/', mockRequestParams);

    expect(fetchMaker)
      .to
      .have
      .been.calledWith(`test/${mockUrl}`, defaultRequestParams);
  });

  it('Should call fetchMaker without a custom base url and ' +
    'an urlMaker function.', () => {
    const fetchMaker = sinon.spy();

    request(undefined, undefined, undefined, fetchMaker)(
      'test/', mockRequestParams);

    expect(fetchMaker)
      .to
      .have
      .been.calledWith('test/', defaultRequestParams);
  });

  it('Should call fetchMaker with a custom base url ' +
    'an urlMaker function and payload.', () => {
    const mockPayload = {
      data: 'test',
    };

    const mockModel = {
      data: String,
    };

    const mockRequestParam = {
      ...defaultRequestParams,
      body: { data: 'test' },
    };

    const fetchMaker = sinon.spy();

    request(mockUrlMaker, undefined, mockModel, fetchMaker)(
      mockRequestParams, mockPayload);

    expect(fetchMaker)
      .to
      .have
      .been.calledWith(`${defaultBaseUrl}${mockUrl}`, mockRequestParam);
  });

  it('Should throw inside the promise if any of the validations fail.',
    (done) => {
      request()('bogus-base-url')
        .catch((err) => {
          expect(err).to.be.an('error');
          done();
        });
    });
});

describe('getRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(getRequest()).to.be.a('function');
  });
});

describe('postRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(postRequest()).to.be.a('function');
  });
});

describe('putRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(putRequest()).to.be.a('function');
  });
});

describe('patchRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(patchRequest()).to.be.a('function');
  });
});
